
#include "config.h"
#include <sys/param.h>
#include "ieee802_11.h"



int wlan_header_len(u_int16_t fc)
{
    int hdrlen = MGMT_HDRLEN;
    switch (FC_TYPE(fc)) {
    case T_DATA:
	if (FC_TO_DS(fc) && FC_FROM_DS(fc)) 
	    hdrlen = MAX_DATA_HEADER_LEN;
	break;
	
    case T_CTRL:
	switch (FC_SUBTYPE(fc)) {
	case CTRL_CTS:
	case CTRL_ACK:
	    hdrlen = CTRL_ACK_HDRLEN;
	    break;
	default:
	    hdrlen = CTRL_RTS_HDRLEN;
	    break;
	}
	break;
    }
    return hdrlen;
}


const char* wlan_type_str(u_int16_t fc)
{
    switch (FC_TYPE(fc)) {
    case T_DATA:
	return "data";
    case T_MGMT:
	return "mgmt";
    case T_CTRL:
	return "ctrl";
    default:
	return "unkn";
    }
}

const char* wlan_subtype_str(u_int16_t fc)
{
    int stype = FC_SUBTYPE(fc);
    switch (FC_TYPE(fc)) {
    case T_DATA:
	return wlan_data_str(stype);
    case T_MGMT:
	return wlan_mgmt_str(stype);
    case T_CTRL:
	return wlan_ctrl_str(stype);
    default:
	return "unkn  ";
    }
}


const char* wlan_ctrl_str(int stype)
{
    switch (stype) {
    case CTRL_PS_POLL:
	return "pspol ";
    case CTRL_RTS:
	return "rts   ";
    case CTRL_CF_END:
	return "cfend ";
    case CTRL_END_ACK:
	return "cfeack";
    case CTRL_CTS:
	return "cts   ";
    case CTRL_ACK:
	return "ack   ";
    default:
	return "unkn  ";
    }
}


const char* wlan_mgmt_str(int stype)
{
    switch (stype) {
    case ST_ASSOC_REQUEST:
	return "assreq";
    case ST_ASSOC_RESPONSE:
	return "assrsp";
    case ST_REASSOC_REQUEST:
	return "rasreq";
    case ST_REASSOC_RESPONSE:
	return "rasrsp";
    case ST_PROBE_REQUEST:
	return "prbreq";
    case ST_PROBE_RESPONSE:
	return "prbrsp";
    case ST_BEACON:
	return "beacon";
    case ST_ATIM:
	return "trfind";
    case ST_DISASSOC:
	return "disass";
    case ST_AUTH:
	return "auth  ";
    case ST_DEAUTH:
	return "deauth";
    default:
	return "unkn  ";
    }
}


const char* wlan_data_str(int stype)
{
    switch (stype) {
    case ST_DATA:
	return "data ";
    case ST_DATA_CF_ACK:
	return "d+ack";
    case ST_DATA_CF_POLL:
	return "d+pol";
    case ST_DATA_CF_ACK_POLL:
	return "d+pak";
    case ST_NULL_FUNC:
	return "null ";
    case ST_CF_ACK:
	return "cfack";
    case ST_CF_POLL:
	return "cfpol";
    case ST_CF_ACK_POLL:
	return "cfpak";
    default:
	return "unkn ";
    }
}

