/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */

//***** APWARE

#include "config.h"
#include <stdlib.h>
#include <stdio.h>
#include <linux/if_ether.h>   /* The L2 protocols */

#include "wlansniff.h"
#include "wlansniff_airo.h"
#include "wlansniff_hostap.h"
#include "wlansniff_orinoco.h"

int
frame_type(const wlan_frame_t *frm)
{
    if (frm) {
	// The frist 2 bytes is always FC. 
	return FC_TYPE(frm->data_hdr->fc);
    }
    return -1;
}


int 
frame_subtype(const wlan_frame_t* frm)
{
    if (frm) {
	// The frist 2 bytes is always FC. 
	return FC_SUBTYPE(frm->data_hdr->fc);
    }
    return -1;
}


sniff_drv_func*
get_drv_func(int type)
{
    if (type < 0 || type > (SNIFF_TYPE_LEN - 1)) {
	return NULL;
    }
    return &sniff_f[type];
}

sniff_rssi_conv_func
get_rssi_conv_func(int type)
{
    if(type < SNIFF_TYPE_LEN)
	return sniff_rssi_f[type];
    else
	return sniff_rssi_f[SNIFF_UNKNOWN];
}


/*****************************************************************
 * Driver specific print routines 
 ******************************************************************
 */


void 
sniff_init_hostap(char *data, struct wlan_frame_t *frm) 
{
    // data[0] is prism_header type
    if (data[0] == 3) {
	    frm->not_encapped  = 1;
    }

    if (data[0] == 1 || data[0] == 3) {
	    struct linux_wlan_ng_prism_hdr *hdr;
	    hdr = (struct linux_wlan_ng_prism_hdr *)(&data[1]);
	    
	    frm->time = host_to_le32(hdr->mactime.data);
	    frm->signal =  host_to_le32(hdr->signal.data);
	    frm->noise = host_to_le32(hdr->noise.data);
	    frm->rate = host_to_le32(hdr->rate.data * 5);
    }
    else {

    }
} 

void 
sniff_print_hostap(char *data, struct wlan_frame_t *frm, FILE *file) 
{
    struct linux_wlan_ng_prism_hdr *hdr;
    hdr = (struct linux_wlan_ng_prism_hdr *)(&data[1]);
    //fprintf(file, " 0x%x", hdr->status);
}


void 
sniff_init_airo(char *data, struct wlan_frame_t *frm) 
{
    struct linux_wlansniff_airo_rxhdr *hdr;
    hdr = (struct linux_wlansniff_airo_rxhdr *)data;
    frm->time   = host_to_le32(hdr->airo_hdr.rxtime);
    frm->signal = hdr->airo_hdr.rssi[AIRO_SIGNAL_STRENGTH];
    frm->qual   = hdr->airo_hdr.rssi[AIRO_SIGNAL_QUALITY];
    frm->noise  = 0;
    frm->freq   = hdr->airo_hdr.freq; /* TODO: need to verify conversion value */
    frm->rate   = hdr->airo_hdr.rate;
}


void 
sniff_print_airo(char *data, struct wlan_frame_t *frm, FILE *file) 
{
    struct linux_wlansniff_airo_rxhdr *hdr;
    hdr = (struct linux_wlansniff_airo_rxhdr *)data;
    fprintf(file, " airo rxassoc %08x plcp %08x",
	    host_to_le32(hdr->airo_hdr.rxassoc_rsv),
	    host_to_le32(hdr->airo_hdr.plcp));
}

void 
sniff_init_orinoco(char *data, struct wlan_frame_t *frm) 
{
    struct hfa384x_rx_frame *hdr;
    hdr = (struct hfa384x_rx_frame *)data;
    frm->time   = host_to_le32(hdr->time);
    frm->signal = hdr->signal;
    frm->qual   = 0;
    frm->noise  = hdr->silence;
    frm->rate   = hdr->rate;
}


void 
sniff_print_orinoco(char *data, struct wlan_frame_t *frm, FILE *file) 
{
    struct hfa384x_rx_frame *hdr;
    hdr = (struct hfa384x_rx_frame *)data;
    fprintf(file, " orinoco status %04x rxflow %02x reserved %08x",
	    host_to_le16(hdr->status),
	    hdr->rxflow,
	    host_to_le32(hdr->reserved));
}


sniff_drv_func  sniff_f[SNIFF_TYPE_LEN] = {
    { NULL, NULL },
    { sniff_init_hostap, sniff_print_hostap },
    { sniff_init_airo, sniff_print_airo }, 
    { sniff_init_orinoco, sniff_print_orinoco }, 
};

int sniff_no_conv(int rssi) { return rssi; }
int sniff_airo_rssi_conv(int rssi) { return rssi - 256; }
int sniff_orinoco_rssi_conv(int rssi) { return rssi - 0x95; }

sniff_rssi_conv_func sniff_rssi_f[SNIFF_TYPE_LEN] = {
    sniff_no_conv,
    sniff_no_conv,
    sniff_airo_rssi_conv,
    sniff_orinoco_rssi_conv,    
};
