#ifndef WLANSNIFF_H
#define WLANSNIFF_H

//***** APWARE

#include <stdio.h>  
#include "wlansniff_common.h" 
#include "ieee802_11.h"
#include "pcap-wlansniff.h"

struct sniff_drv_func;


typedef struct wlan_frame_t {
    // Device specific physical layer info
    u8  freq;    // channel number
    u8  rate;    // data rate in 100kbps
    u8  signal;  // received signal strength (in dbm?)
    u8  noise;   // noise level (in dbm?)
    u8  qual;    // signal quality
    u64 time;    // timestamp, 1 microsecond resolution

    int  fc;     // frame_control 
    u16 fcs;     // MAC CRC 
    int  not_encapped;// frame is not encapsulated in a LLC frame (non-monitor mode)
    struct sniff_drv_func *drv;
    char *drv_data;

    union {
	struct data_header_t *data_hdr;
	struct mgmt_header_t *mgmt_hdr;
	struct ctrl_rts_t *rts_hdr;
	struct ctrl_cts_t *cts_hdr;
	struct ctrl_end_t *end_hdr;
	struct ctrl_ps_poll_t *poll_hdr;
    };
} wlan_frame_t;


extern int frame_type(const wlan_frame_t *frm);
extern int frame_subtype(const wlan_frame_t *frm);


// sniff_header related (defined here to avoid including in sniffproto.h
// which was meant to include only constants and sniff_header.

typedef void (*sniff_func_init)(char *data, wlan_frame_t *frm);
typedef void (*sniff_func_print)(char *data, wlan_frame_t *frm,
				 FILE *file);
typedef int (*sniff_rssi_conv_func)(int rssi);

typedef struct sniff_drv_func {
    sniff_func_init init;
    sniff_func_print print;
} sniff_drv_func;

extern sniff_drv_func sniff_f[SNIFF_TYPE_LEN];
extern sniff_rssi_conv_func sniff_rssi_f[SNIFF_TYPE_LEN];

extern sniff_drv_func* get_drv_func(int type); 
extern sniff_rssi_conv_func get_rssi_conv_func(int type); 

#endif
